@@ -1,9 +1,9 @@
 package Data::Sah::Filter;
 
 our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
-our $DATE = '2020-02-10'; # DATE
+our $DATE = '2020-02-11'; # DATE
 our $DIST = 'Data-Sah-Filter'; # DIST
-our $VERSION = '0.002'; # VERSION
+our $VERSION = '0.003'; # VERSION
 
 use strict 'subs', 'vars';
 use warnings;
@@ -29,15 +29,15 @@ This is mostly for testing. Normally the filter rules will be used from
 
 _
     args => {
-        filter_names => {
-            schema => ['array*', of=>'str*'],
-        },
+        %Data::Sah::FilterCommon::gen_filter_args,
     },
     result_naked => 1,
 };
 sub gen_filter {
     my %args = @_;
 
+    my $rt = $args{return_type} // 'val';
+
     my $rules = Data::Sah::FilterCommon::get_filter_rules(
         %args,
         compiler=>'perl',
@@ -56,19 +56,45 @@ sub gen_filter {
             }
         }
 
+        my $code_filter = "";
+        my $has_defined_tmp;
+        for my $rule (@$rules) {
+            if ($rule->{meta}{might_fail}) {
+                if ($rt eq 'val') {
+                    $code_filter .= "    my \$tmp; " unless $has_defined_tmp++;
+                    $code_filter .= "    \$tmp = $rule->{expr_filter}; return undef if \$tmp->[0]; ";
+                } else {
+                    $code_filter .= "    \$data = $rule->{expr_filter}; return \$data if \$data->[0]; ";
+                }
+            } else {
+                if ($rt eq 'val') {
+                    $code_filter .= "    \$data = $rule->{expr_filter}; ";
+                } else {
+                    $code_filter .= "    \$data = [undef, $rule->{expr_filter}]; ";
+                }
+            }
+        }
+
         $code = join(
             "",
             $code_require,
             "sub {\n",
             "    my \$data = shift;\n",
             "    unless (defined \$data) {\n",
-            "        return undef;\n",
+            "        ", ($rt eq 'val' ? "return undef;" :
+                             "return [undef, undef];" # str_errmsg+val
+                         ), "\n",
             "    }\n",
-            (map { "    \$data = $_->{expr_filter};\n" } @$rules),
+            $code_filter, "\n",
+            "    \$data;\n",
             "}",
         );
     } else {
-        $code = 'sub { $_[0] }';
+        if ($rt eq 'val') {
+            $code = 'sub { $_[0] }';
+        } else {
+            $code = 'sub { [undef, $_[0]] }';
+        }
     }
 
     if ($Log_Filter_Code) {
@@ -97,7 +123,7 @@ Data::Sah::Filter - Filtering for Data::Sah
 
 =head1 VERSION
 
-This document describes version 0.002 of Data::Sah::Filter (from Perl distribution Data-Sah-Filter), released on 2020-02-10.
+This document describes version 0.003 of Data::Sah::Filter (from Perl distribution Data-Sah-Filter), released on 2020-02-11.
 
 =head1 SYNOPSIS
 
@@ -156,6 +182,17 @@ indicates required keys):
 
 =over
 
+=item * might_fail => bool
+
+Whether coercion might fail, e.g. because of invalid input. If set to 1,
+C<expr_filter> key that the C<filter()> routine returns must be an expression
+that returns an array (envelope) of C<< (error_msg, data) >> instead of just
+filtered data. Error message should be a string that is set when filtering fails
+and explains why. Otherwise, if filtering succeeds, the string should be set to
+undefined value.
+
+This is used for filtering rules that act as a data checker.
+
 =item * expr_filter => str
 
 Expression in the target language to actually convert data.
@@ -213,7 +250,9 @@ Arguments ('*' denotes required arguments):
 
 =over 4
 
-=item * B<filter_names> => I<array[str]>
+=item * B<filter_names>* => I<array[str]>
+
+=item * B<return_type> => I<str> (default: "val")
 
 
 =back
