@@ -1,9 +1,9 @@
 package Data::Sah::FilterCommon;
 
 our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
-our $DATE = '2020-02-10'; # DATE
+our $DATE = '2020-02-11'; # DATE
 our $DIST = 'Data-Sah-Filter'; # DIST
-our $VERSION = '0.002'; # VERSION
+our $VERSION = '0.003'; # VERSION
 
 use 5.010001;
 use strict 'subs', 'vars';
@@ -11,14 +11,26 @@ use warnings;
 
 our %SPEC;
 
+our %common_args = (
+    filter_names => {
+        schema => ['array*', of=>'str*'],
+        req => 1,
+    },
+);
+
+our %gen_filter_args = (
+    %common_args,
+    return_type => {
+        schema => ['str*', in=>['val', 'errstr+val']],
+        default => 'val',
+    },
+);
+
 $SPEC{get_filter_rules} = {
     v => 1.1,
     summary => 'Get filter rules from filter rule modules',
     args => {
-        filter_names => {
-            schema => ['array*', of=>'str*'],
-            req => 1,
-        },
+        %common_args,
         compiler => {
             schema => 'str*',
             req => 1,
@@ -37,7 +49,10 @@ sub get_filter_rules {
     my $prefix = "Data::Sah::Filter::$compiler\::";
 
     my @rules;
-    for my $filter_name (@{ $args{filter_names} }) {
+    for my $entry (@{ $args{filter_names} }) {
+        my $filter_name = ref $entry eq 'ARRAY' ? $entry->[0] : $entry;
+        my $filter_gen_args = ref $entry eq 'ARRAY' ? $entry->[1] : undef;
+
         my $mod = $prefix . $filter_name;
         (my $mod_pm = "$mod.pm") =~ s!::!/!g;
         require $mod_pm;
@@ -50,6 +65,7 @@ sub get_filter_rules {
         }
         my $rule = &{"$mod\::filter"}(
             data_term => $dt,
+            (args => $filter_gen_args) x !!$filter_gen_args,
         );
         $rule->{name} = $filter_name;
         $rule->{meta} = $filter_meta;
@@ -74,7 +90,7 @@ Data::Sah::FilterCommon - Common stuffs for Data::Sah::Filter and Data::Sah::Fil
 
 =head1 VERSION
 
-This document describes version 0.002 of Data::Sah::FilterCommon (from Perl distribution Data-Sah-Filter), released on 2020-02-10.
+This document describes version 0.003 of Data::Sah::FilterCommon (from Perl distribution Data-Sah-Filter), released on 2020-02-11.
 
 =head1 FUNCTIONS
 
